# macOS
|Python Version|brew python@3.9|brew python@3.10|brew python@3.11|
|:------------:|:-------------:|:--------------:|:--------------:|
|     3.9      |       X       |                |                |
|     3.10     |               |       X        |                |
|     3.11     |               |                |       X        |

[macOS](https://en.wikipedia.org/wiki/MacOS) is a pseudo-BSD-based general-purpose operating system, developed and commercially supported by Apple Inc.

macOS is released approximately every year, with security updates historically available for ~3 years after release.

*NOTE: macOS is not "free" (as in "user freedom") software*.

## Installing Python on macOS: Supported Method
**First, take a deep breath**.
*This is not always entirely simple. But we promise to walk you through it!*

If you've previously installed Python, **uninstall it thoroughly before starting**.

### 1. Installing `brew` - The Missing Package Manager for macOS
[Homebrew](https://brew.sh/) is a package manager for your Mac.
Think of it as an **app store for nerds** (yes you are, M\*. Engineer!)

1. Downlod the latest `.pkg` installer from Homebrew: <https://github.com/Homebrew/brew/releases>
2. Follow the instructions.

Good job! Let's get Python now.

### 2. Installing `python`
*Now for the scary bit*.

1. Open up Spotlight, and search for "Terminal". Open it!
2. Type the following command: `brew install python@3`
3. Press Enter!

Now, sit back as `brew` does everything for you.
That's it!

### 3. Updating `python`
The best thing about any *app store* ($\equiv$ package managers) is, that it can update stuff for you.

1. Open up Spotlight, and search for "Terminal". Open it!
2. Type the following command: `brew update`, then press ENTER.
	- This **checks for new versions** of any `brew`-managed software.
3. Type the following command: `brew upgrade`, then press ENTER.
	- This **actually updates** all `brew`-managed software, if updates are available.

That's it!

### Disclaimer
**When you update Python, your code may break**.
Additionally, your code may simply not work for anyone using a slightly different Python version and/or slightly different packages.

See [our article on Python projects](../../projects/basics/README.md) to learn how to prevent this in a collaboration-friendly way.



# Important Information
Here are some things to keep in mind when using Python on macOS.

## Don't Use pip (without venv)
Many websites will encourage you to run things like `sudo pip install`.

**Don't**.

Seriously, it is **very easy to break macOS with pip**.

When using Python packages on macOS, you **must** isolate them from your system.
The easiest way to do this is with the built-in `venv` ("Virtual Environments").
See [our article on venv](../packages/venv.md) for help.



# Unsupported Installation Methods
We **strongly suggest** using the method described above.
Here we'll briefly describe some other installation methods, and explain why we don't suggest them.

## Built-In Python
Some versions of macOS come with a built-in version of Python.
These are optimized for internal use by your operating system.

We don't suggest using it for the following reasons:
- **It cannot be updated**. Attempting to is nearly guaranteed to *break macOS*.
- The installation is tailored to the internal needs of the macOS system, *not* to the needs of users. This can cause small, but significant, issues.

## python.org Installer
An installer for Python is available from python.org(<https://www.python.org/downloads/macos/>)

We don't suggest it for the following reasons:
- Finding what to download can be confusing, and there is only one right option.
- The installer asks some confusing questions, which can lead to very significant differences in how the install works.
- Updates must be installed manually.
- Generally speaking, *software downloaded from websites shouldn't be trusted*. Rather, a trusted package manager / app store should be preferred.



# Reference
- Python Documentation on macOS: <https://www.python.org/downloads/macos/>
